# メカナムホイール サンプルプログラム（L4A6ZG用）

メカナムホイールのサンプルプログラム（NUCLEO-L4A6ZG用）です。

動作確認を目的としたもので、UART2またはUART3からの入力に応じて、特定の動作を行います。

なお、UART2、UART3はこのサンプルプログラムではいずれもフロー制御あり(RTS/CTS)になっています。

## モータの動作確認

起動後（またはリセット後）、メインボード上のLED6=MOTOR\_ENが1秒間点灯するので、その間にメインボード上のSW3=USER\_SWを押すとモータの動作確認モードになります。

このモードでは、台車が各方向に動く（および回転を行う）、という動作を繰り返します。その他の操作（以下のUART2からの動作確認）は受け付けません。

モータの動作確認モードを抜けるには、リセットボタンを押すか、いったん電源をOFFにします。

## UART2の動作

UART2では、さまざまな機能の動作確認を行います。

### UART2の動作確認

UART2に文字の'0'〜'9'（0x30〜0x39）の入力があった時、'A'〜'J'を返します。

### モータの動作（個別）

UART2に'm'（0x6D）の入力があった時、モータの正転および逆転を各モータについて行い、各動作について以下の値を整数で順に返信します。

- 現在のループ（0〜8）。
- エンコーダでのカウント値を40で割ったもの（-32768〜32767、2バイト）。
- 速度カウンタでのカウント値を10で割ったもの（0〜65535、2バイト）。

また、最後に'T'を返信します。合計で3×8+1=25バイトのデータを返すことになります。

ここでは、以下の２つの値が等しくなることが期待されます。

- 「エンコーダでのカウント値」の絶対値。
- 「速度カウンタでのカウント値」を2倍にしたもの。

なお、実際にはタイミングにより値が1だけずれることがありますが、2以上のずれは異常と考えられます。

### モータの動作（PWM）

UART2に'M'（0x4D）の入力があった時、全モータの正転および逆転をデューティ比を変化させながら行い、各デューティ比について以下の値を整数で順に返信します。

- 現在のループ（0〜39、1バイト、カウント開始のタイミングで送信）。
- エンコーダでのカウント値（-32768〜32767、4x2バイト=8バイト）。
- 速度カウンタでのカウント値（0〜65535、4x2バイト=8バイト）。

また、最後に'T'を返信します。合計で40x8+1=641バイトのデータを返すことになります。

### DACの設定

UART2に'd'（0x64）の入力があった時、続いて入力される2バイトの数値（各0〜255）で2つのDACチャンネルの出力値を設定します。

DACの設定値は、1バイトの入力値を16倍にした値（0〜4080）となります。

### ADC値の取得

UART2に'a'（0x61）の入力があった時、4つのADC入力端子の値を各1バイトの数値（0〜255）で返信します。返信値に対して、電圧は以下のように計算できます。

- (返信値)×3.3/255 [V]

### バッテリー関連ADC値の取得

UART2に'A'（0x41）の入力があった時、VehicleとComputerのバッテリー電圧を各1バイトの数値（0〜255）で返信します。返信値に対して、電圧は以下のように計算できます。

- (返信値)×6.6/255 [V]

### I2C通信

UART2に'c'（0x63）の入力があった時、I2C通信のテストを行います。UART2では、'c'に続いて以下の値を入力します。

- アドレスを1ビット左シフトしたもの（1バイト）
- 受信データの長さ（0〜64、1バイト）
- 送信データの長さ（0〜64、1バイト）
- 送信データ（「送信データの長さ」で指定したバイト数）

以上を受信してI2C通信をした後、I2Cで受信したデータを「受信データの長さ」で指定したバイト数だけ返します。

### SPI通信

UART2に'p'（0x70）の入力があった時、SPI通信のテストを行います。UART2では、'p'に続いて以下の値を入力します。

- SPIチャンネル（'1'、'2'、'3'、'4'のいずれか。1バイト）
- 送受信データの長さ（0〜64、1バイト）
- 送信データ（「送受信データの長さ」で指定したバイト数）

以上を受信してSPI通信をした後、SPIで受信したデータを返します（長さは「送受信データの長さ」で指定したバイト数になります）。

なお、このサンプルプログラムではSPI1（'1'）、SPI2（'2'）はスレーブ、SPI3（'3'）はマスターです。SPI4（'4'）を指定した場合はSPI1を指定したと見做されます。

### RS485通信

UART2に'e'（0x65）の入力があった時、RS485通信のテストを行います。UART2では、'e'に続いて以下の値を入力します。

- 受信データの長さ（0〜64、1バイト）
- 送信データの長さ（0〜64、1バイト）
- 送信データ（「送信データの長さ」で指定したバイト数）

以上を受信してRS485通信をした後、RS485で受信したデータを「受信データの長さ」で指定したバイト数だけ返します。

### CAN通信

UART2に'f'（0x66）の入力があった時、CAN通信のテストを行います。UART2では、f'に続いて以下の値を入力します。

- 受信データの長さ（0〜8、1バイト）
- 送信データの長さ（0〜8、1バイト）
- 送信データ（「送信データの長さ」で指定したバイト数）

以上を受信してCAN通信をした後、CANで受信したデータを「受信データの長さ」で指定したバイト数だけ返します。

### ユーザスイッチのカウント数の取得

UART2に'u'（0x75）の入力があった時、ユーザスイッチが何回押されたかのカウント数を1バイト（0〜255）で返します。

### 静的変数のカウント数の取得

UART2に's'（0x73）の入力があった時、それ以前に's'入力が何回あったかのカウント数を1バイト（0〜255）で返します。

### その他

UART2への上記以外の入力は無視されます。

## UART3の動作

UART3は、文字の'0'〜'9'（0x30〜0x39）の入力があった時に、'a'〜'j'を返します。その他の入力値は無視されます。
