#include "bmx055.h"
#include <stdio.h>

void i2c_write(I2C_HandleTypeDef *i2c, int addr, uint8_t *buf, int sendLen) {
	uint8_t ret = HAL_I2C_Master_Transmit(i2c, addr, buf, sendLen, 3000);
	if(ret==0) return;
	printf("write(0x%02x):", addr>>1);
	for(int i=0;i<sendLen;i++) {
		printf(" 0x%02x", buf[i]);
	}
	printf(" (%d)\r\n", ret);
	while(1);
}

void i2c_read(I2C_HandleTypeDef *i2c, int addr, uint8_t *buf, int recvLen) {
	uint8_t ret = HAL_I2C_Master_Receive(i2c, addr|1, buf, recvLen, 3000);
	if(ret==0) return;
	printf("read(0x%02x):", addr>>1);
	for(int i=0;i<recvLen;i++) {
		printf(" 0x%02x", buf[i]);
	}
	printf(" (%d)\r\n", ret);
	while(1);
}

// I2C初期設定
void bmx055_i2c_init(I2C_HandleTypeDef *i2c) {
    //csag = 1;
    //csm = 1;

	setbuf(stdout, NULL);

    const int addr_a = 0x19<<1;
    const int addr_g = 0x69<<1;
    const int addr_m = 0x13<<1;
    uint8_t cmd[2] = {0, 0};

    printf("HAL_ERROR: %d\r\n", HAL_ERROR);
    printf("HAL_OK   : %d\r\n", HAL_OK   );
    printf("HAL_BUSY : %d\r\n", HAL_BUSY );

    // 加速度の設定
    printf("Acc settings\r\n");
    HAL_Delay(1);
    cmd[0] = 0x0F; // Select PMU_Range register
    cmd[1] = 0x03; // Range = +/- 2g
    i2c_write(i2c, addr_a, cmd, 2);
    //HAL_Delay(10);

    cmd[0] = 0x10; // Select PMU_BW register
    cmd[1] = 0x08; // Bandwidth = 7.81 Hz
    i2c_write(i2c, addr_a, cmd, 2);
    //HAL_Delay(10);

    cmd[0] = 0x11; // Select PMU_LPW register
    cmd[1] = 0x00; // Normal mode, Sleep duration = 0.5ms
    i2c_write(i2c, addr_a, cmd, 2);
    //HAL_Delay(10);

    // ジャイロの設定
    printf("Gyro settings\r\n");
    cmd[0] = 0x0F; // Select Range register
    cmd[1] = 0x04; // Full scale = +/- 125 degree/s
    i2c_write(i2c, addr_g, cmd, 2);
    //HAL_Delay(100);

    cmd[0] = 0x10; // Select Bandwidth register
    cmd[1] = 0x07; // ODR = 100 Hz
    i2c_write(i2c, addr_g, cmd, 2);
    //HAL_Delay(100);

    cmd[0] = 0x11; // Select LPM1 register
    cmd[1] = 0x00; // Normal mode, Sleep duration = 2ms
    i2c_write(i2c, addr_g, cmd, 2);
    //HAL_Delay(100);

    // 磁気の設定
    printf("Mag settings\r\n");
    cmd[0] = 0x4B; // Select Mag register
    cmd[1] = 0x83; // Soft reset
    //i2c_write(i2c, addr_m, cmd, 2);
    i2c_write(i2c, addr_m, &cmd[0], 1);
    i2c_write(i2c, addr_m, &cmd[1], 1);
    //HAL_Delay(100);

    cmd[0] = 0x4B; // Select Mag register
    cmd[1] = 0x01; // Soft reset
    i2c_write(i2c, addr_m, cmd, 2);
    HAL_Delay(1);

    cmd[0] = 0x4C; // Select Mag register
    cmd[1] = 0x00; // Normal Mode, ODR = 10 Hz
    i2c_write(i2c, addr_m, cmd, 2);

    cmd[0] = 0x4E; // Select Mag register
    cmd[1] = 0x84; // X, Y, Z-Axis enabled
    i2c_write(i2c, addr_m, cmd, 2);

    cmd[0] = 0x51; // Select Mag register
    cmd[1] = 0x04; // No. of Repetitions for X-Y Axis = 9
    i2c_write(i2c, addr_m, cmd, 2);

    cmd[0] = 0x52; // Select Mag register
    cmd[1] = 0x16; // No. of Repetitions for Z-Axis = 15
    i2c_write(i2c, addr_m, cmd, 2);

}

// I2C加速度取得
void bmx055_i2c_get_a(I2C_HandleTypeDef *i2c, double a[3]) {
    const int addr_a = 0x19<<1;
    uint8_t cmd[1] = {0};
    uint8_t dat[2] = {0, 0};
    for(int i=0;i<3;i++) {
        cmd[0] = 2+2*i;
        i2c_write(i2c, addr_a, cmd, 1);
        i2c_read(i2c, addr_a|1, &dat[0], 1);
        cmd[0] = 2+2*i+1;
        i2c_write(i2c, addr_a, cmd, 1);
        i2c_read(i2c, addr_a|1, &dat[1], 1);
        a[i] = (dat[1]*256+dat[0])*(4.0/65536);
        if(a[i]>2.0) a[i] -= 4.0;
    }
}

// I2Cジャイロ取得
void bmx055_i2c_get_g(I2C_HandleTypeDef *i2c, double g[3]) {
    const int addr_g = 0x69<<1;
    uint8_t cmd = 0;
    uint8_t dat[2] = {0, 0};
    for(int i=0;i<3;i++) {
        cmd = 2+2*i;
        i2c_write(i2c, addr_g, &cmd, 1);
        i2c_read(i2c, addr_g|1, &dat[0], 1);
        cmd = 2+2*i+1;
        i2c_write(i2c, addr_g, &cmd, 1);
        i2c_read(i2c, addr_g|1, &dat[1], 1);
        g[i] = (dat[1]*256+dat[0])*(250.0/65536);
        if(g[i]>125.0) g[i] -= 250.0;
    }
}

// I2C磁気取得
void bmx055_i2c_get_m(I2C_HandleTypeDef *i2c, double m[3]) {
	const int addr_m = 0x13 << 1;
	uint8_t cmd = 0;
	uint8_t dat[2] = { 0, 0 };
	for (int i = 0; i < 4; i++) {
		cmd = 0x42 + 2 * i;
		i2c_write(i2c, addr_m, &cmd, 1);
		i2c_read(i2c, addr_m | 1, &dat[0], 1);
		cmd = 0x42 + 2 * i + 1;
		i2c_write(i2c, addr_m, &cmd, 1);
		i2c_read(i2c, addr_m | 1, &dat[1], 1);
		if (i > 2)
			continue;
		int32_t val = dat[1] * 256L + dat[0];
		if (val > 32767L)
			val -= 65536L;
		if (i == 0 || i == 1) {
			m[i] = 1.0 * (val >> 3);
		} else if (i == 2) {
			m[i] = 1.0 * (val >> 1);
		}
	}
}

void bmx055_i2c_test(I2C_HandleTypeDef *i2c) {

	// 初期化
	bmx055_i2c_init(i2c);
	HAL_Delay(300);

	while (1) {

		printf("OK?\r\n");

		// 加速度取得
		double a[3] = { 0, 0, 0 };
		bmx055_i2c_get_a(i2c, a);

		// ジャイロ取得
		double g[3] = { 0, 0, 0 };
		bmx055_i2c_get_g(i2c, g);

		// 磁気取得
		double m[3] = { 0, 0, 0 };
		bmx055_i2c_get_m(i2c, m);

		printf("a=[%f %f %f]\r\n", a[0], a[1], a[2]);
		printf("g=[%f %f %f]\r\n", g[0], g[1], g[2]);
		printf("m=[%8.0f %8.0f %8.0f]\r\n", m[0], m[1], m[2]);

		HAL_Delay(1000);
	}
}
